#ifndef CGUL_BASE64_P_H
#define CGUL_BASE64_P_H

#include "cgul_base64.h"

CGUL_BEGIN_C

/* Only in odd circumstance would this need to be changed. */
typedef unsigned char base64_u8;

static const base64_u8 BASE64_PAD_CHAR = 0xfe;
static const base64_u8 BASE64_ILLEGAL_CHAR = 0xff;

/**
 * Data encapsulated by the <code>cgul_base64</code> class.
 */
struct cgul_base64 {

    /*
     * Encode variables.
     */

    /** Internal buffer for holding encoded data. */
    base64_u8* ebuf;

    /** Count of only the converted bytes in "ebuf". */
    size_t ebuf_size;

    /** Count of all the bytes in "ebuf". */
    size_t ebuf_size_max;

    /**
     * Maximum number of encoded characters to output before embedding
     * a '\n' character.
     */
    size_t line_length_max;

    /** The current length of the last line encoded in "buf". */
    size_t line_length_curr;

    /**
     * The number of bytes remaining to be processed between calls to
     * encode data.
     */
    size_t e_n_bytes_remaining;

    /** Value of the byte 0 remaining to be processed between calls. */
    base64_u8 er0;

    /** Value of the byte 1 remaining to be processed between calls. */
    base64_u8 er1;

    /*
     * Decode variables.
     */

    /** Internal buffer for holding decoded data. */
    base64_u8* dbuf;

    /** Count of only the converted bytes in "dbuf". */
    size_t dbuf_size;

    /** Count of all the bytes in "dbuf". */
    size_t dbuf_size_max;

    /**
     * If true, to not ignore invalid base64 characters when decoding
     * including '\n' characters.
     */
    int strict_decode;

    /**
     * The number of bytes remaining to be processed between calls to
     * encode data.
     */
    size_t d_n_bytes_remaining;

    /** Value of the byte 0 remaining to be processed between calls. */
    base64_u8 dr0;

    /** Value of the byte 1 remaining to be processed between calls. */
    base64_u8 dr1;

    /** Value of the byte 2 remaining to be processed between calls. */
    base64_u8 dr2;

    /** Value of the byte 3 remaining to be processed between calls. */
    base64_u8 dr3;

};

CGUL_END_C

#endif  /*  CGUL_BASE64_P_H  */
