#include "cgul_block_reader_file_p.h"
#include "cgul_libc.h"

/**
 * Virtual Function Table for the <code>cgul_block_reader_file</code> class.
 */
struct cgul_block_reader_file__static_vtable {

    /* Declare */

    /**
     * Virtual methods derived from <code>cgul_block_reader</code>.
     */
    struct cgul_block_reader__vtable br_vtable;

} cgul_block_reader_file__static_vtable_instance = {

    /* Static Initialization */

    /* br_vtable */
    {
        (cgul_block_reader__virtual_delete_t)
        cgul_block_reader_file__delete,

        (cgul_block_reader__virtual_read_next_block_t)
        cgul_block_reader_file__read_next_block,

        (cgul_block_reader__virtual_get_presentation_name_t)
        cgul_block_reader_file__get_presentation_name,

        (cgul_block_reader__virtual_set_presentation_name_t)
        cgul_block_reader_file__set_presentation_name,
    },

};

/**
 * Common code to avoid memory leaks when setting the presentation name.
 *
 * @param[in,out] cex c-style exception
 * @param[in] brfile <code>cgul_block_reader_file</code> instance
 * @param[in] presentation_name presentation name
 */
static void
cgul_block_reader_file__set_presentation_name__common(
    cgul_exception_t* cex,
    cgul_block_reader_file_t brfile,
    const char* presentation_name)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* Clear the old presentation name. */
    if (brfile->presentation_name) {
        free(brfile->presentation_name);
        brfile->presentation_name = NULL;
    }

    /* Set the new presentation name. */
    brfile->presentation_name = cgul_libc__strdup(cex, presentation_name);
    if (*cex) {
        goto out;
    }

 out:
    return;
}

/**
 * Initialize or reinitialize the <code>cgul_block_reader_file</code>
 * object <code>brfile</code>.  If an error occurs, an exception is
 * thrown.
 *
 * @param[in,out] cex c-style exception
 * @param[in] brfile <code>cgul_block_reader_file</code> instance
 */
static void
cgul_block_reader_file__reinitialize(cgul_exception_t* cex,
                                     cgul_block_reader_file_t brfile)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* super -- initialized in constructor */

    /* Close the underlying file but only if we own it. */
    if (brfile->f) {
        if (brfile->we_own_f) {
            fclose(brfile->f);
        }
        brfile->f = NULL;
    }

    /* we_own_f */
    brfile->we_own_f = 0;

    /* Free the file name. */
    if (brfile->fname) {
        free(brfile->fname);
        brfile->fname = NULL;
    }

    /* Set the presentation name to its default value. */
    cgul_block_reader_file__set_presentation_name__common(cex, brfile, "FILE");
    if (*cex) {
        goto out;
    }

 out:
    return;
}

cgul_block_reader_t
cgul_block_reader_file__as_block_reader(cgul_exception_t* cex,
                                        cgul_block_reader_file_t brfile)
{
    cgul_block_reader_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* Return pointer to embedded super-class. */
    rv = &brfile->super;

 out:
    return rv;
}

cgul_block_reader_file_t
cgul_block_reader_file__new(cgul_exception_t* cex)
{
    cgul_block_reader_file_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Allocate. */
    rv = (cgul_block_reader_file_t)cgul_libc__calloc(cex, 1, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    /* super */
    rv->super.vptr =
        &cgul_block_reader_file__static_vtable_instance.br_vtable;

    /* Initialize the new object. */
    cgul_block_reader_file__reinitialize(cex, rv);
    if (*cex) {
        goto out;
    }

 out:

    /* Delete partially instantiated objects. */
    if (*cex && rv) {
        cgul_block_reader_file__delete(rv);
        rv = NULL;
    }

    return rv;
}

cgul_block_reader_file_t
cgul_block_reader_file__new_from_fname(cgul_exception_t* cex,
                                       const char* fname)
{
    cgul_block_reader_file_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Create a default object. */
    rv = cgul_block_reader_file__new(cex);
    if (*cex) {
        goto out;
    }

    /* Open "fname". */
    cgul_block_reader_file__open_fname(cex, rv, fname);
    if (*cex) {
        goto out;
    }

 out:

    /* Delete partially instantiated objects. */
    if (*cex && rv) {
        cgul_block_reader_file__delete(rv);
        rv = NULL;
    }

    return rv;
}

cgul_block_reader_file_t
cgul_block_reader_file__new_from_file(cgul_exception_t* cex,
                                      FILE* f)
{
    cgul_block_reader_file_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Create a default object. */
    rv = cgul_block_reader_file__new(cex);
    if (*cex) {
        goto out;
    }

    /* Open "f". */
    cgul_block_reader_file__open_file(cex, rv, f);
    if (*cex) {
        goto out;
    }

 out:

    /* Delete partially instantiated objects. */
    if (*cex && rv) {
        cgul_block_reader_file__delete(rv);
        rv = NULL;
    }

    return rv;
}

void
cgul_block_reader_file__delete(cgul_block_reader_file_t brfile)
{
    cgul_exception_t local = NULL;

    if (brfile) {

        /* Close the underlying file. */
        cgul_block_reader_file__close(&local, brfile);
        if (local) {
            /* Printing a warning is the best we can do here because
             * we are inside the destructor. */
            const char* msg = cgul_exception__get_message(local);
            fprintf(stderr,
                    "*** Warning: cgul_block_reader_file__delete: %s\n",
                    msg);
            cgul_exception__delete(local);
            local = NULL;
        }

        /* presentation_name */
        if (brfile->presentation_name) {
            free(brfile->presentation_name);
        }

        /* Free the instance itself. */
        free(brfile);
    }
}

void
cgul_block_reader_file__open_fname(cgul_exception_t* cex,
                                   cgul_block_reader_file_t brfile,
                                   const char* fname)
{
    int reset_object_on_error = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* If an error occurs, reset the brfile object to its default values. */
    reset_object_on_error = 1;

    /* Sanity check. */
    if (!fname) {
        *cex = cgul_exception__new("cgul_block_reader_file__open_fname: "
                                   "file name is NULL");
        goto out;
    }

    /* Close the current file before attempting to open the new file. */
    cgul_block_reader_file__close(cex, brfile);

    /* Open the underlying file. */
    brfile->f = cgul_libc__fopen(cex, fname, "rb");
    if (*cex) {
        goto out;
    }

    /* Take ownership of the file. */
    brfile->we_own_f = 1;

    /* Duplicate the file name. */
    assert(brfile->fname == NULL);
    brfile->fname = cgul_libc__strdup(cex, fname);
    if (*cex) {
        goto out;
    }

    /* By default, the presentation name should match the file name. */
    cgul_block_reader_file__set_presentation_name__common(cex, brfile, fname);
    if (*cex) {
        goto out;
    }

 out:

    /* Clean up. */
    if (*cex && reset_object_on_error) {
        cgul_exception_t local = NULL;
        cgul_block_reader_file__reinitialize(&local, brfile);
        if (local) {
            /* An exception is already pending the best we can do
             * is print a warning. */
            const char* msg = cgul_exception__get_message(local);
            fprintf(stderr,
                    "Warning: cgul_block_reader_file__open_fname: "
                    "full object reset failed after failing "
                    "to open file: %s: %s",
                    fname, msg);
            cgul_exception__delete(local);
            local = NULL;
        }
    }

    return;
}

void
cgul_block_reader_file__open_file(cgul_exception_t* cex,
                                  cgul_block_reader_file_t brfile,
                                  FILE* f)
{
    int reset_object_on_error = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* If an error occurs, reset the brfile object to its default values. */
    reset_object_on_error = 1;

    /* Sanity check. */
    if (!f) {
        *cex = cgul_exception__new("cgul_block_reader_file__open_file: "
                                   "file is NULL");
        goto out;
    }

    /* Close the current file before attempting to open the new file. */
    cgul_block_reader_file__close(cex, brfile);

    /* Use the file provided by the user but do not take ownership of it. */
    brfile->f = f;
    brfile->we_own_f = 0;

 out:

    /* Clean up. */
    if (*cex && reset_object_on_error) {
        cgul_exception_t local = NULL;
        cgul_block_reader_file__reinitialize(&local, brfile);
        if (local) {
            /* An exception is already pending the best we can do
             * is print a warning. */
            const char* msg = cgul_exception__get_message(local);
            fprintf(stderr,
                    "Warning: cgul_block_reader_file__open_file: "
                    "full object reset failed after failing to open file: %s",
                    msg);
            cgul_exception__delete(local);
            local = NULL;
        }
    }

    return;
}

void
cgul_block_reader_file__close(cgul_exception_t* cex,
                              cgul_block_reader_file_t brfile)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* Reinitialize. */
    cgul_block_reader_file__reinitialize(cex, brfile);

 out:
    return;
}

size_t
cgul_block_reader_file__read_next_block(cgul_exception_t* cex,
                                        cgul_block_reader_file_t brfile,
                                        void* buf,
                                        size_t block_size)
{
    size_t rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);
    assert(buf);

    /* Sanity check. */
    if (!brfile->f) {
        *cex = cgul_exception__new(
            "cgul_block_reader_file__read_next_block: "
            "Attempting to read before opening the file.");
        goto out;
    }

    /* Sanity check. */
    if (!buf) {
        goto out;
    }

    /* Read the blocks.  Because fread() returns the number of
    *  _blocks_ read, we have to use a block size of 1 and a count of
    *  "block_size" to get fread() to report the number of _bytes_
    *  that were read. */
    rv = cgul_libc__fread(cex, buf, 1, block_size, brfile->f);
    if (*cex) {
        goto out;
    }

 out:
    return rv;
}

const char*
cgul_block_reader_file__get_fname(cgul_exception_t* cex,
                                  cgul_block_reader_file_t brfile)
{
    const char* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* Throw an exception if no file is open. */
    if (!brfile->f) {
        *cex = cgul_exception__new("cgul_block_reader_file__get_file: "
                                   "no file is open");
        goto out;
    }

    /* Return the file name or NULL. */
    rv = brfile->fname;

 out:
    return rv;
}

const char*
cgul_block_reader_file__get_presentation_name(
    cgul_exception_t* cex,
    cgul_block_reader_file_t brfile)
{
    const char* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* Sanity check. */
    if (!brfile->f) {
        *cex = cgul_exception__new(
            "cgul_block_reader_file__get_presentation_name: "
            "no input file");
        goto out;
    }

    /* Return the presentation name. */
    assert(brfile->presentation_name);
    rv = brfile->presentation_name;

 out:
    return rv;
}

void
cgul_block_reader_file__set_presentation_name(
    cgul_exception_t* cex,
    cgul_block_reader_file_t brfile,
    const char* presentation_name)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    /* Sanity check. */
    if (!brfile->f) {
        *cex = cgul_exception__new(
            "cgul_block_reader_file__set_presentation_name: "
            "no input file");
        goto out;
    }

    /* Set the presentation name. */
    cgul_block_reader_file__set_presentation_name__common(
        cex, brfile, presentation_name);
    if (*cex) {
        goto out;
    }

 out:
    return;
}

FILE*
cgul_block_reader_file__get_file(cgul_exception_t* cex,
                                 cgul_block_reader_file_t brfile)
{
    void* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(brfile);

    /* Throw an exception if no file is open. */
    if (!brfile->f) {
        *cex = cgul_exception__new("cgul_block_reader_file__get_file: "
                                   "no file is open");
        goto out;
    }

    /* Return the file. */
    rv = brfile->f;

 out:
    return (FILE*)rv;
}
