#ifndef CGUL_CONVSPEC_P_H
#define CGUL_CONVSPEC_P_H

#include "cgul_convspec.h"

CGUL_BEGIN_C

/**
 * Data encapsulated by the <code>cgul_convspec</code> class.
 */
struct cgul_convspec {

    /**
     * Whether the '#' flag is present in the conversion
     * specification.
     */
    char flag_hash;

    /**
     * Whether the '0' flag is present in the conversion
     * specification.
     */
    char flag_zero;

    /**
     * Whether the '-' flag is present in the conversion
     * specification.
     */
    char flag_dash;

    /**
     * Whether the ' ' flag is present in the conversion
     * specification.
     */
    char flag_space;

    /**
     * Whether the '+' flag is present in the conversion
     * specification.
     */
    char flag_plus;

    /**
     * Whether the field_width is present.
     */
    char is_field_width_present;

    /**
     * If the field width is present, the user can specify a '*'
     * character instead of a number.  In this case, the caller should
     * ignore the <code>field_width</code> value (below).  Instead,
     * the field width is given by the next argument on the stack.
     */
    char is_field_width_on_stack;

    /**
     * Minimum field width.
     */
    int field_width;

    /**
     * Whether the precision is present.
     */
    char is_precision_present;

    /**
     * If the precision is present, the user can specify a '*'
     * character instead of a number.  In this case, the caller should
     * ignore the <code>precision</code> value (below).  Instead, the
     * precision is given by the next argument on the stack.
     */
    char is_precision_on_stack;

    /**
     * Precision.
     */
    int precision;

    /**
     * Whether the length modifier is present.
     */
    char is_length_modifier_present;

    /**
     * Length modifier.  Because the length modifier can be things
     * like "ll" or "hh" in C99, we use a string to hold it.
     */
    cgul_string_t length_modifier;

    /**
     * Conversion specifier.  For example, this is the 'd' character
     * in "%d".
     */
    char cspec;

};

CGUL_END_C

#endif  /*  CGUL_CONVSPEC_P_H  */
