#include "cgul_hash_node_p.h"
#include "cgul_hash_p.h"
#include "cgul_libc.h"

cgul_hash_node_t
cgul_hash_node__new(cgul_exception_t* cex)
{
    cgul_hash_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    rv = (cgul_hash_node_t)cgul_libc__calloc(cex, 1, sizeof(*rv));
    if (*cex) {
        goto out;
    }

 out:

    /* Clean up if an exception is thrown. */
    if (*cex && rv) {
        cgul_hash_node__delete(rv);
        rv = NULL;
    }

    return rv;
}

void
cgul_hash_node__delete(cgul_hash_node_t n)
{
    if (n) {
        free(n);
    }
}

const void*
cgul_hash_node__get_key(cgul_exception_t* cex,
                        cgul_hash_node_t n)
{
    const void* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = n->key;

 out:
    return rv;
}

void
cgul_hash_node__set_key(cgul_exception_t* cex,
                        cgul_hash_node_t n,
                        const void* key)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);
    assert(key);

    n->key = key;

 out:
    return;
}

void*
cgul_hash_node__get_value(cgul_exception_t* cex,
                          cgul_hash_node_t n)
{
    void* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = (void*)n->value;

 out:
    return rv;
}

void
cgul_hash_node__set_value(cgul_exception_t* cex,
                          cgul_hash_node_t n,
                          const void* value)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    n->value = value;

 out:
    return;
}

cgul_hash_node_t
cgul_hash_node__get_prev(cgul_exception_t* cex,
                         cgul_hash_node_t n)
{
    cgul_hash_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = n->older;

 out:
    return rv;
}

cgul_hash_node_t
cgul_hash_node__get_next(cgul_exception_t* cex,
                         cgul_hash_node_t n)
{
    cgul_hash_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = n->younger;

 out:
    return rv;
}

cgul_hash_node_t
cgul_hash_node__get_older(cgul_exception_t* cex,
                          cgul_hash_node_t n)
{
    cgul_hash_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = n->older;

 out:
    return rv;
}

cgul_hash_node_t
cgul_hash_node__get_younger(cgul_exception_t* cex,
                            cgul_hash_node_t n)
{
    cgul_hash_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = n->younger;

 out:
    return rv;
}
