#include "cgul_hchar.h"

const cgul_hchar_t CGUL_HCHAR__EMPTY_STRING[1] = {0};

size_t
cgul_hchar__hcslen(cgul_exception_t* cex,
                   const cgul_hchar_t* hs)
{
    size_t rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(hs);

    /* Count all the wide characters before the null terminator. */
    while (hs[rv] != CGUL_HCHAR__NUL) {
        ++rv;
    }

 out:
    return rv;
}

void
cgul_hchar__hcscpy(cgul_exception_t* cex,
                   cgul_hchar_t* dst,
                   const cgul_hchar_t* src)
{
    size_t i = 0;
    size_t src_length = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(dst);
    assert(src);

    /* Don't do anything if dst and src point to the same memory. */
    if (dst == src) {
        goto out;
    }

    /* Determine the length of the string to copy. */
    src_length = cgul_hchar__hcslen(cex, src);

    /* Copy src to dst.  This is a little slow, but it allows "dst"
     * and "src" to overlap.  Note that we use "i <= src_length"
     * instead of the usual "i < src_length" in order to copy the null
     * terminator too. */
    for (i = 0 ; i <= src_length ; ++i) {
        dst[i] = src[i];
    }

 out:
    return;
}

int
cgul_hchar__hcscmp(const cgul_hchar_t* hs1,
                   const cgul_hchar_t* hs2)
{
    int rv = 0;

    assert(hs1);
    assert(hs2);

    /* Iterate over hs1 and hs2 searching for the first character that
     * differs. */
    for ( ; *hs1 && *hs2 ; ++hs1, ++hs2) {

        /* If this character in hs1 is less than its counterpart in
         * hs2, the hs1 string is less than the hs2 string. */
        if (*hs1 < *hs2) {
            rv = -1;
            goto out;
        }

        /* If this character in hs1 is greater than its counterpart in
         * hs2, the hs1 string is greater than the hs2 string. */
        if (*hs1 > *hs2) {
            rv = 1;
            goto out;
        }
    }

    /* If the loop above reached the end of both strings, they are equal. */
    if ((*hs1 == CGUL_HCHAR__NUL) && (*hs2 == CGUL_HCHAR__NUL)) {
        rv = 0;
        goto out;
    }

    /* If the loop above reached the end hs1 only, hs1 is less than hs2. */
    if (*hs1 == CGUL_HCHAR__NUL) {
        rv = -1;
        goto out;
    }

    /* The loop above must have reached the end hs2 in which case hs1
     * is greater than hs2. */
    assert(*hs2 == CGUL_HCHAR__NUL);
    rv = 1;

 out:
    return rv;
}

int
cgul_hchar__isspace(cgul_exception_t* cex,
                    cgul_hchar_t hc)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    switch (hc) {
    case (cgul_hchar_t)' ':
    case (cgul_hchar_t)'\t':
    case (cgul_hchar_t)'\n':
    case (cgul_hchar_t)'\r':
    case (cgul_hchar_t)'\f':
    case (cgul_hchar_t)'\v':
        rv = 1;
        break;
    default:
        break;
    }

 out:
    return rv;
}
