#include "cgul_list_node_p.h"
#include "cgul_libc.h"

/*
 * This method is used as the "restructor" for the "cgul_cache" of
 * nodes in the "cgul_list".  "restructor" methods are defined in
 * "cgul_cache" to not throw exceptions.
 *
 * While the signature for this method does not exactly match the
 * signature defined for "restructor" methods, it can be safely
 * typecast to match.
 */
void
cgul_list_node__initialize(cgul_list_node_t n)
{
    assert(n);
    memset(n, 0, sizeof(*n));
}

cgul_list_node_t
cgul_list_node__new(cgul_exception_t* cex)
{
    cgul_list_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    rv = (cgul_list_node_t)cgul_libc__malloc(cex, sizeof(*rv));
    if (*cex) {
        goto out;
    }

    cgul_list_node__initialize(rv);

 out:

    /* Free partially constructed objects. */
    if (*cex && rv) {
        cgul_list_node__delete(rv);
        rv = NULL;
    }

    return rv;
}

void
cgul_list_node__delete(cgul_list_node_t n)
{
    if (n) {
        free(n);
    }
}

void*
cgul_list_node__get_value(cgul_exception_t* cex,
                          cgul_list_node_t n)
{
    void* rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = (void*)n->value;

 out:
    return rv;
}

void
cgul_list_node__set_value(cgul_exception_t* cex,
                          cgul_list_node_t n,
                          const void* value)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    n->value = value;

 out:
    return;
}

cgul_list_node_t
cgul_list_node__get_prev(cgul_exception_t* cex,
                         cgul_list_node_t n)
{
    cgul_list_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = n->prev;

 out:
    return rv;
}

void
cgul_list_node__set_prev(cgul_exception_t* cex,
                         cgul_list_node_t n,
                         cgul_list_node_t prev)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    n->prev = prev;

 out:
    return;
}

cgul_list_node_t
cgul_list_node__get_next(cgul_exception_t* cex,
                         cgul_list_node_t n)
{
    cgul_list_node_t rv = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    rv = n->next;

 out:
    return rv;
}

void
cgul_list_node__set_next(cgul_exception_t* cex,
                         cgul_list_node_t n,
                         cgul_list_node_t next)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(n);

    n->next = next;

 out:
    return;
}
