/**
 * The program merges multiple bdf font files into a single bdf font
 * file that is written to standard output.  The fonts should appear
 * on the command in order of priority.  So, if an encoding appears in
 * multiple input BDFs, the encoding that appears in the output BDF is
 * the one that comes from the input BDF that is listed earlier on the
 * command line. <p>
 *
 * This program appends a "i" to the name of each BDF to try to find a
 * BDF index file for that BDF.  For example, if the fonts name is
 * "foo.bdf", this program tries to open the "foo.bdfi" index file.
 * If BDF index files are used, it can greatly speed up
 * processing. <p>
 *
 * @note The FONT line that appears in the output BDF is taken
 * verbatim from the FONT line from the BDF that is listed first on
 * the command line.  It is recommended that user review this line to
 * make sure it is still appropriate for the merged font.
 *
 * @file merge multiple bdf font files into a single bdf font file
 */

#include <cstdlib>
#include <iostream>
#include <sstream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    cgul_bdf_cxx bdf;

    /* Verify the command line. */
    if (argc < 2) {
        cerr << "Usage: main_bdf_merge <font1.bdf> [<font2.bdf> ...]" << endl;
        exit(1);
    }

    try {

        /* Merge each BDF file into the "bdf" object. */
        for (i = 1 ; i < argc ; ++i) {
            bdf.load_explicit(argv[i], NULL);
        }

        /* Print the result to stdout. */
        bdf.write_file(stdout);

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
