#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    FILE* f = NULL;
    size_t rcount = 0;
    unsigned char* block = NULL;
    size_t block_size = 16384;
    cgul_crc32_incr_t crc = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    block = cgul_libc__calloc(&local, 1, block_size);
    crc = cgul_crc32_incr__new_appends_size(
              &local, 0x04c11db7, 0, 0xffffffff, 0, 0);
    if (local) {
        goto out;
    }

    /* Validate the command line. */
    if (argc != 2) {
        fprintf(stderr, "Usage: main <file>\n");
        goto out;
    }

    /* Open the file. */
    f = cgul_libc__fopen(&local, argv[1], "rb");
    if (local) {
        goto out;
    }

    /* Iterate over each block in the file. */
    for (;;) {

        /* Read the next block. */
        rcount = fread(block, 1, block_size, f);
        if (rcount == 0) {
            if (ferror(f)) {
                local = cgul_exception__new("fread: %m");
                goto out;
            }
            /* EOF */
            break;
        }

        /* Update incremental checksum. */
        cgul_crc32_incr__update_block(&local, crc, block, rcount);

        /* Print the incremental checksum. */
        cgul_libc__printf(&local, "%lu\n", cgul_crc32_incr__crc(&local, crc));
    }

    /* Print the final checksum. */
    cgul_libc__printf(&local, "%lu\n", cgul_crc32_incr__crc(&local, crc));

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (f) {
        fclose(f);
    }
    if (crc) {
        cgul_crc32_incr__delete(crc);
    }
    if (block) {
        free(block);
    }

    return rv;
}
