#include <iostream>
#include "cgul_cxx.h"

using namespace std;

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    FILE* f = NULL;
    size_t rcount = 0;
    unsigned char* block = NULL;
    size_t block_size = 16384;
    cgul_crc32_incr_cxx* crc = NULL;

    try {

        /* Allocate. */
        block = (unsigned char*)cgul_libc_cxx::calloc(1, block_size);
        crc = new cgul_crc32_incr_cxx(0x04c11db7, 0, 0xffffffff, 0, 0);

        /* Validate the command line. */
        if (argc != 2) {
            fprintf(stderr, "Usage: main <file>\n");
            goto out;
        }

        /* Open the file. */
        f = cgul_libc_cxx::fopen(argv[1], "rb");

        /* Iterate over each block in the file. */
        for (;;) {

            /* Read the next block. */
            rcount = cgul_libc_cxx::fread(block, 1, block_size, f);
            if (rcount == 0) {
                /* EOF */
                break;
            }

            /* Update incremental checksum. */
            crc->update_block(block, rcount);

            /* Print the incremental checksum. */
            cout << crc->crc() << endl;
        }

        /* Print the final checksum. */
        cout << crc->crc() << endl;

    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

 out:

    /* Clean up. */
    if (f) {
        fclose(f);
    }
    if (crc) {
        delete crc;
    }
    if (block) {
        free(block);
    }

    return rv;
}
