#include <iostream>
#include <stdexcept>
#include <string>
#include "cgul_cxx.h"

using namespace std;

/**
 * In-memory CSV "file."
 */
static string memory = "\
header1,header2,header3,header4,header5,header6\n\
Now is the,time for all,good men to,come to the,aid of their,country.\n\
\"Hello, World!\n\
Goodbye, Cruel World!\",\"He said, \"\"Are you talking to me?\"\"\",,,,\n\
\"\"\"Hello, World!\"\"\",,,,,IN-MEMORY!!!\n\
";

/**
 * Print the rows and columns in the CSV file <code>csv</code>.
 *
 * @param[in] csv CSV file
 * @param[in] strm output stream
 */
static void
print(cgul_csv_cxx& csv,
      ostream& strm)
{
    unsigned long int i = 0;
    unsigned long int n = 0;
    const char* field = NULL;

    /* Iterate over each record. */
    while (csv.load_next()) {

        /* Test getting the field by the header for its column. */
        field = csv.lookup_by_header("header1");
        if (field) {
            strm << "field[\"header1\"] = " << field << endl;
        }

        /* Iterate over each field. */
        n = csv.get_number_of_fields();
        for (i = 0 ; i < n ; ++i) {
            field = csv.lookup(i);
            strm << "field[" << i << "] = " << field << endl;
        }

        strm << "=====" << endl;
    }
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;

    try {

        /* Open the CSV file and print. */
        cgul_crlf_file_cxx f("main_csv_input.csv");
        cgul_csv_cxx csv(f);
        print(csv, cout);
        cout << endl;
        cout << endl;

        /* Open CSV "file" in memory and print. */
        f.open_memory(memory.c_str(), memory.length());
        csv.open_crlf_file(f);
        print(csv, cout);
        cout << endl;
        cout << endl;

        /* Open the file again and print. */
        f.open_fname("main_csv_input.csv");
        csv.open_crlf_file(f);
        print(csv, cout);
        cout << endl;
        cout << endl;

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
