#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    unsigned long int n = 0;
    const char* field = NULL;
    const char* header = NULL;
    cgul_csv_t csv = NULL;
    cgul_rbtree_t headers = NULL;
    cgul_rbtree_node_t node = NULL;
    cgul_exception_t local = NULL;

    /* Create a new cgul_csv instance. */
    csv = cgul_csv__new_from_fname(&local,
                                   "main_csv_tsv_input.tsv",
                                   1 /* is_block_buffered */);
    if (local) {
        goto out;
    }

    /* Use the tab character as the separator. */
    cgul_csv__set_separator(&local, csv, '\t');

    /* Get the headers for the cgul_csv instance. */
    headers = cgul_csv__get_headers(&local, csv);

    /* Iterate over each row in the csv file. */
    while (cgul_csv__load_next(&local, csv)) {

        /* Check for errors. */
        if (local) {
            goto out;
        }

        /* Iterate over each field. */
        n = cgul_csv__get_number_of_fields(&local, csv);
        for (i = 0 ; i < n ; ++i) {
            field = cgul_csv__lookup(&local, csv, i);
            printf("field[%lu] = %s\n", i, field);
        }

        /* Iterate again but use the headers to do the lookup. */
        node = cgul_rbtree__get_front(&local, headers);
        for ( ; node ; node = cgul_rbtree_node__get_next(&local, node)) {
            header = cgul_rbtree_node__get_key(&local, node);
            field = cgul_csv__lookup_by_header(&local, csv, header);
            if (field) {
                printf("field[\"%s\"] = %s\n", header, field);
            }
        }

    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    if (csv) {
        cgul_csv__delete(csv);
    }

    return rv;
}
