#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    char* ptr = NULL;
    size_t length = 0;
    unsigned long int cksum = 0;
    cgul_crc32_t crc32 = NULL;
    cgul_exception_t local = NULL;

    /* Verify command line. */
    if (argc != 2) {
        fprintf(stderr, "Usage: main_libc_load_file <file>\n");
        goto out;
    }

    /* Allocate. */
    crc32 = cgul_crc32__new_appends_size(
                &local, 0x04c11db7, 0, 0xffffffff, 0, 0);
    if (local) {
        goto out;
    }

    /* Load the file into memory. */
    cgul_libc__load_fname(&local, argv[1], &ptr, &length);
    if (local) {
        goto out;
    }

    /* Calculate and print the file's checksum. */
    cksum = cgul_crc32__crc_block(&local, crc32, ptr, length);
    printf("%lu %lu %s\n", cksum, (unsigned long)length, argv[1]);
    

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (ptr) {
        free(ptr);
    }
    if (crc32) {
        cgul_crc32__delete(crc32);
    }

    return rv;
}
