#include "cgul.h"

static int
print_key(cgul_exception_t* cex,
          const void* key,
          void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("key = %s\n", (char*)key);
    rv = 1;

 out:
    return rv;
}

static int
print_value(cgul_exception_t* cex,
            void* value,
            void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("value = %d\n", *(int*)value);
    rv = 1;

 out:
    return rv;
}

static int
print_pair(cgul_exception_t* cex,
           const void* key,
           void* value,
           void* data)
{
    int rv = 0;

    assert(cex);
    if (*cex) {
        goto out;
    }

    printf("key = %s  value = %d\n", (char*)key, *(int*)value);
    rv = 1;

 out:
    return rv;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    char* key = NULL;
    int* value = NULL;
    cgul_multimap_t mm = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    mm = cgul_multimap__new(&local, (cgul_multimap__compare_t)strcmp);
    if (local) {
        goto out;
    }

    /* Populate the multimap.  Each key gets two values. */
    for (i = 0 ; i < 10 ; i += 2) {

        /* Allocate a new key and value to add to the multimap. */
        key = cgul_libc__calloc(&local, 1, sizeof(char) * 2);
        value = cgul_libc__calloc(&local, 1, sizeof(*value));
        if (local) {
            goto out;
        }
        /* Assign values to the newly allocated memory. */
        *key = 'a' + i/2;
        *value = i;
        /* Insert the key/value pair into the multimap. */
        cgul_multimap__insert(&local, mm, key, value, NULL);
        if (local) {
            goto out;
        }

        /* Allocate a new value to add to the multimap. */
        value = cgul_libc__calloc(&local, 1, sizeof(*value));
        if (local) {
            goto out;
        }
        /* Assign values to the newly allocated memory. */
        *value = i + 1;
        /* Insert the key/value pair into the multimap. */
        cgul_multimap__insert(&local, mm, key, value, NULL);
        if (local) {
            goto out;
        }
    }

    /* Print the keys in forward and then reverse order. */
    cgul_multimap__foldl_keys(&local, mm, print_key, NULL);
    cgul_multimap__foldr_keys(&local, mm, print_key, NULL);

    puts("=====");

    /* Print the values in forward and then reverse order. */
    cgul_multimap__foldl_values(&local, mm, 0, print_value, NULL);
    cgul_multimap__foldr_values(&local, mm, 0, print_value, NULL);

    puts("=====");

    /* Print the key/value pairs in forward and then reverse order. */
    cgul_multimap__foldl_pairs(&local, mm, 0, print_pair, NULL);
    cgul_multimap__foldr_pairs(&local, mm, 0, print_pair, NULL);

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (mm) {
        cgul_multimap__free_keys(mm);
        cgul_multimap__free_values(mm);
        cgul_multimap__delete(mm);
    }

    return rv;
}
