#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_random_lfg32_t prng = NULL;

#ifdef CGUL_UINT64
    cgul_uint32_t x32 = 0;
    cgul_uint32_t y32 = 0;
    cgul_uint32_t hi32 = 0;
    cgul_uint32_t lo32 = 0;
    cgul_uint64_t i = 0;
    cgul_uint64_t z64 = 0;
#endif

    cgul_exception_t local = NULL;

#ifndef CGUL_UINT64
    local = cgul_exception__new("cgul_uint64_t not defined!");
    goto out;
#else

    /* Allocate. */
    prng = cgul_random_lfg32__new(&local);
    if (local) {
        goto out;
    }

    /* Test forever. */
    for (;;) {

        /* Get the next two 32-bit multiplicands. */
        x32 = cgul_random_lfg32__next(&local, prng);
        y32 = cgul_random_lfg32__next(&local, prng);

        /* Calculate the 64-bit product using native multiplication. */
        z64 = (cgul_uint64_t)x32 * y32;

        /* Calculate the 64-bit product using cgul_multiply64(). */
        cgul_multiply64(&local, x32, y32, &hi32, &lo32);

#if 0
        printf("%u * %u = %lu\n", x32, y32, z64);
        printf("%u * %u = %#0x%08x", x32, y32, hi32, lo32);
#endif

        /* Verify. */
        if ((z64 >> 32) != hi32) {
            local = cgul_exception__new("%u * %u = %#0x%08x",
                                        x32, y32, hi32, lo32);
            goto out;
        }
        if ((z64 & 0xffffffff) != lo32) {
            local = cgul_exception__new("%u * %u = %#0x%08x",
                                        x32, y32, hi32, lo32);
            goto out;
        }

        /* Print progress. */
        if ((++i & 0xffffff) == 0) {
            printf("i = %lu\n", (unsigned long)i);
        }
    }

#endif

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (prng) {
        cgul_random_lfg32__delete(prng);
    }

    return rv;
}
