#include "cgul.h"
#include <sys/time.h>

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    int ascending_order = 1;
    size_t v_count = 1000000;
    unsigned char* v = NULL;
    struct timeval start = {0};
    struct timeval stop = {0};
    cgul_random_cmwc32_t prng = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    v = cgul_libc__calloc(&local, v_count, sizeof(*v));
    prng = cgul_random_cmwc32__new(&local);
    if (local) {
        goto out;
    }

    /* Initialize. */
    for (i = 0 ; i < v_count ; ++i) {
        //v[i] = v_count - i - 1;
        v[i] = (unsigned char)
            cgul_random_cmwc32__next_in_range(&local, prng, 0x100);
    }

#if 0
    /* Print the array before it is sorted. */
    printf("===== Before\n");
    for (i = 0 ; i < v_count ; ++i) {
        printf("%hhu\n", v[i]);
    }
    fflush(stdout);
#endif

    /* Sort the array. */
    gettimeofday(&start, NULL);
    cgul_radix_sort_uchar(&local, v, v_count, ascending_order);
    if (local) {
        goto out;
    }
    gettimeofday(&stop, NULL);

#if 1
    {
        double t0 = start.tv_sec + start.tv_usec / 1000000.0;
        double t1 = stop.tv_sec + stop.tv_usec / 1000000.0;
        fprintf(stderr, "Sort Time: %f\n", t1 - t0);
    }
#endif

#if 1
    /* Verify the list is sorted. */
    for (i = 1 ; i < v_count ; ++i) {
        if (ascending_order) {
            if (v[i - 1] > v[i]) {
                break;
            }
        } else {
            if (v[i - 1] < v[i]) {
                break;
            }
        }
    }
    if ((v_count == 0) || (i == v_count)) {
        fprintf(stderr, "Sort verified!\n");
    } else {
        fprintf(stderr, "Sort failed!\n");
    }
#endif

#if 0
    /* Print the array after it is sorted. */
    ///printf("===== After\n");
    for (i = 0 ; i < v_count ; ++i) {
        printf("%hhu\n", v[i]);
    }
    fflush(stdout);
#endif

 out:

    /* Handle exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (prng) {
        cgul_random_cmwc32__delete(prng);
    }
    if (v) {
        free(v);
    }

    return rv;
}
