#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

static int
compare_pointers(char* p1,
                 char* p2)
{
    return p1 - p2;
}

static int
remove_node_cb(cgul_rbtree_cxx* t,
               cgul_rbtree_node_cxx* node,
               void* data)
{
    t->remove_node(node, NULL, NULL);
    return 1;
}

static int
print_node_cb(cgul_rbtree_cxx* t,
              cgul_rbtree_node_cxx* node,
              void* data)
{
    cout << node->get_key() << endl;
    return 1;
}

int
main(int argc, char* argv[])
{
    int rv = 0;
    char* p = NULL;

    try {

        cgul_rbtree_cxx t((cgul_rbtree_cxx::compare_t)&compare_pointers);
        t.set_cache_size(10);

        /* 1 */
        for (p = 0x0 ; p < (char*)10 ; ++p) {
            t.insert(p, NULL);
        }

        /* Remove the nodes between 0x6 and 0x8 inclusive. */
        cgul_rbtree_node_cxx* first = t.find((void*)0x6);
        cgul_rbtree_node_cxx* last = t.find((void*)0x8);
        if (first && last) {
            t.traverse_range(first, last, &remove_node_cb, NULL);
        }

        /* Print the remaining nodes. */
        t.traverse(&print_node_cb, (void*)0x7a);

        while (t.get_size() > 0) {
            t.remove_front(NULL, NULL);
        }

        /* 2 */
        for (p = 0x0 ; p < (char*)10 ; ++p) {
            t.insert(p, NULL);
        }

        while (t.get_size() > 0) {
            t.remove_back(NULL, NULL);
        }

        /* 3 -- but without the removes in order to test dtor. */
        for (p = 0x0 ; p < (char*)10 ; ++p) {
            t.insert(p, NULL);
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
