#include <cassert>
#include <cstring>
#include <iostream>
#include <stdexcept>
#include "cgul_cxx.h"

using namespace std;

/**
 * This method prints the key/value pair for <code>key</code>.
 *
 * @param[in] stanza stanza
 * @param[in] key key
 */
static void
print_key_value_pair(const cgul_stanza_cxx& stanza,
                     const char* key)
{
    const char* value = NULL;

    value = stanza.lookup(key);
    if (value == NULL) {
        value = "<invalid key>";
    }
    cout << key << " ==> " << value << endl;
}

int main(int argc, char* argv[])
{
    int rv = 0;
    cgul_stanza_cxx stanza;

    try {

        /*
         * It is not necessary, but we want to test to make sure the
         * allowed_keys and the mandatory_keys code is working.
         *
         * NOTE: Below we have two different cgul_rbtree objects for
         * the allowed_keys list and the mandatory_keys list.
         * Typically, you will pass in the same list for both.
         */

        /* The allowed_keys list is designed to warn when unexpected
         * keys are in a stanza. */
        cgul_rbtree_cxx allowed_keys((cgul_rbtree__compare_t)strcmp);
        allowed_keys.insert("foo", NULL);
        allowed_keys.insert("bar", NULL);
        allowed_keys.insert("baz", NULL);
        allowed_keys.insert("hmm", NULL);

        /* The mandatory_keys list is designed to warn when an
         * expected key is missing from a stanza. */
        cgul_rbtree_cxx mandatory_keys((cgul_rbtree__compare_t)strcmp);
        mandatory_keys.insert("foo", NULL);
        mandatory_keys.insert("bar", NULL);
        mandatory_keys.insert("baz", NULL);

        /*
         * Open the stanza text file.
         */
        stanza.open_fname("main_stanza_input.txt");
#if 0
        stanza.allow_duplicate_keys(1);
#endif
        stanza.set_allowed_keys(&allowed_keys);
        stanza.set_mandatory_keys(&mandatory_keys);

    /* Iterate over each stanza. */
        while (stanza.load_next()) {
            print_key_value_pair(stanza, "foo");
            print_key_value_pair(stanza, "bar");
            print_key_value_pair(stanza, "baz");
            print_key_value_pair(stanza, "hmm");
            cout << "====" << endl;
        }

    } catch (const exception& e) {
        cerr << "*** Error: " << e.what() << endl;
        rv = 1;
    } catch (const cgul_exception_cxx& e) {
        cerr << "*** Error: " << e.get_message() << endl;
        rv = 1;
    }

    return rv;
}
