#include "cgul.h"

/**
 * In-memory stanzas.
 */
static const char* memory = "\
#\n\
# This is the input file used to test main_stanza.c.\n\
#\n\
\n\
# Stanza #1.\n\
foo: value 1\n\
bar: value 2\n\
baz: value 3 (1st line)\n\
  +: value 3 (2nd line)\n\
  +: value 3 (3rd line)\n\
\n\
    # Stanza #2.  Notice that indentation is allowed.\n\
    foo: value 4\n\
    bar: value 5\n\
    baz: value 6\n\
    # The \"hmm\" key is allowed but not mandatory, i.e., it is optional.\n\
    hmm: value 7\n\
\n\
\n\
# Notice that the spaces on the next line do not prevent it from being\n\
# treated as a blank line.\n\
     \n\
\n\
\n\
# Notice also that it is ok to have more than one blank line between\n\
# stanzas.\n\
#\n\
# Stanza #3.\n\
foo: value 8\n\
bar: value 9\n\
baz: value 10 (IN-MEMORY!!!)\n\
";

/**
 * This method prints the key/value pair for <code>key</code>.
 *
 * @param[in] cex c-style exception
 * @param[in] stanza stanza
 * @param[in] key key
 */
static void
print_key_value_pair(cgul_exception_t* cex,
                     cgul_stanza_t stanza,
                     const char* key)
{
    assert(cex);
    if (*cex) {
        goto out;
    }

#if 0
    /* You can either do a normal lookup and let cgul_stanza own the
     * value. */
    {
        const char* value = NULL;

        value = cgul_stanza__lookup(cex, stanza, key);
        if (value == NULL) {
            value = "<invalid key>";
        }
        cgul_libc__printf(cex, "%s ==> %s\n", key, value);
    }

#else

    /* Or, you can take the value for long-term use.  Just remember to
     * free it. */
    {
        char* value = NULL;
        value = cgul_stanza__take_value(cex, stanza, key);
        if (value == NULL) {
            cgul_libc__printf(cex, "%s ==> %s\n", key, "<invalid key>");
        } else {
            cgul_libc__printf(cex, "%s ==> %s\n", key, value);
            /* Remember to free the value. */
            free(value);
        }
    }

#endif

 out:
    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    cgul_stanza_t stanza = NULL;
    cgul_exception_t local = NULL;
    
    /* Allocate. */
    stanza = cgul_stanza__new_from_memory(&local, memory, strlen(memory));
    if (local) {
        goto out;
    }

    /* Iterate over each stanza. */
    while (cgul_stanza__load_next(&local, stanza)) {
        print_key_value_pair(&local, stanza, "foo");
        print_key_value_pair(&local, stanza, "bar");
        print_key_value_pair(&local, stanza, "baz");
        print_key_value_pair(&local, stanza, "hmm");
        cgul_libc__printf(&local, "====\n");
        if (local) {
            goto out;
        }
    }

 out:

    /* Handle exception. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (stanza) {
        cgul_stanza__delete(stanza);
    }

    return rv;
}
