#include "cgul.h"

/**
 * Append the <code>unsigned long</code> values on the half-open range
 * <code>[start, stop)</code> to the vector <code>v</code>.  The
 * client is responsible for calling <code>free()</code> on all the
 * pointers added to <code>v</code>.  If an error occurs, an exception
 * is thrown.
 *
 * @param[in,out] cex c-style exception
 * @param[in] v vector
 * @param[in] start inclusive starting value
 * @param[in] stop exclusive stopping value
 */
static void
main__append_values(cgul_exception_t* cex,
                    cgul_vector_t v,
                    unsigned long int start,
                    unsigned long int stop)
{
    unsigned long int i = 0;
    unsigned long int* n = NULL;

    assert(cex);
    if (*cex) {
        goto out;
    }

    assert(v);

    /* Iterate over the values */
    for (i = start ; i < stop ; ++i) {

        /* Allocate space for the value. */ 
        n = cgul_libc__calloc(cex, 1, sizeof(*n));
        if (*cex) {
            goto out;
        }

        /* Set the value. */
        *n = i;

        /* Append the value to the vector. */
        cgul_vector__push_back(cex, v, n);
        if (*cex) {
            goto out;
        }
    }

 out:
    return;
}

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    unsigned long int i = 0;
    unsigned long int** ns = NULL;
    cgul_vector_t v = NULL;
    cgul_exception_t local = NULL;

    /* Allocate. */
    v = cgul_vector__new(&local);
    if (local) {
        goto out;
    }

    /* Add [0 - 5) to the v. */
    main__append_values(&local, v, 0, 5);
    if (local) {
        goto out;
    }

    /* Append a trailing sentinel. */
    cgul_vector__push_back(&local, v, NULL);
    if (local) {
        goto out;
    }

    /* Take the underlying array. */
    ns = (unsigned long int**)cgul_vector__take_array(&local, v);

    /* Print ns. */
    for (i = 0 ; ns[i] ; ++i) {
        printf("ns[%lu] = %lu\n", i, *ns[i]);
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (v) {
        cgul_vector__free_values(v);
        cgul_vector__delete(v);
    }
    for (i = 0 ; ns[i] ; ++i) {
        free(ns[i]);
    }
    free(ns);

    return rv;
}
