#include "cgul.h"

int
main(int argc,
     char* argv[])
{
    int rv = 0;
    int i = 0;
    cgul_string_t utf8 = NULL;
    cgul_wstring_t utf32 = NULL;
    cgul_exception_t local = NULL;
    const char* tests[] = {
        "", "f", "   f   ", "foo", "   foo  ", "        ", NULL
    };

    /* Allocate. */
    utf8 = cgul_string__new(&local);
    utf32 = cgul_wstring__new(&local);
    if (local) {
        goto out;
    }

    /* Trim Left: Iterate over the test strings. */
    for (i = 0 ; tests[i] ; ++i) {
        /* Convert from UTF-8 to UTF-32. */
        cgul_wstring__set_value(&local, utf32, CGUL_WCHAR__EMPTY_STRING);
        cgul_unicode__mbstowcs(&local,
                               tests[i],
                               strlen(tests[i]),
                               1,  /* skip_bom */
                               utf32);
        /* Trim left. */
        cgul_wstring__trim_left(&local, utf32);
        /* Convert back to UTF-8. */
        cgul_string__set_value(&local, utf8, "");
        cgul_unicode__wcstombs(&local,
                               cgul_wstring__get_value(&local, utf32),
                               cgul_wstring__get_length(&local, utf32),
                               1,  /* skip_bom */
                               utf8);
        /* Check for exceptions. */
        if (local) {
            goto out;
        }
        /* Print result. */
        printf(" trim_left(): \"%s\" --> \"%s\"\n",
               tests[i],
               cgul_string__get_value(&local, utf8));
    }

    printf("=====\n");

    /* Trim Right: Iterate over the test strings. */
    for (i = 0 ; tests[i] ; ++i) {
        /* Convert from UTF-8 to UTF-32. */
        cgul_wstring__set_value(&local, utf32, CGUL_WCHAR__EMPTY_STRING);
        cgul_unicode__mbstowcs(&local,
                               tests[i],
                               strlen(tests[i]),
                               1,  /* skip_bom */
                               utf32);
        /* Trim right. */
        cgul_wstring__trim_right(&local, utf32);
        /* Convert back to UTF-8. */
        cgul_string__set_value(&local, utf8, "");
        cgul_unicode__wcstombs(&local,
                               cgul_wstring__get_value(&local, utf32),
                               cgul_wstring__get_length(&local, utf32),
                               1,  /* skip_bom */
                               utf8);
        /* Check for exceptions. */
        if (local) {
            goto out;
        }
        /* Print result. */
        printf("trim_right(): \"%s\" --> \"%s\"\n",
               tests[i],
               cgul_string__get_value(&local, utf8));
    }

    printf("=====\n");

    /* Trim: Iterate over the test strings. */
    for (i = 0 ; tests[i] ; ++i) {
        /* Convert from UTF-8 to UTF-32. */
        cgul_wstring__set_value(&local, utf32, CGUL_WCHAR__EMPTY_STRING);
        cgul_unicode__mbstowcs(&local,
                               tests[i],
                               strlen(tests[i]),
                               1,  /* skip_bom */
                               utf32);
        /* Trim. */
        cgul_wstring__trim(&local, utf32);
        /* Convert back to UTF-8. */
        cgul_string__set_value(&local, utf8, "");
        cgul_unicode__wcstombs(&local,
                               cgul_wstring__get_value(&local, utf32),
                               cgul_wstring__get_length(&local, utf32),
                               1,  /* skip_bom */
                               utf8);
        /* Check for exceptions. */
        if (local) {
            goto out;
        }
        /* Print result. */
        printf("      trim(): \"%s\" --> \"%s\"\n",
               tests[i],
               cgul_string__get_value(&local, utf8));
    }

 out:

    /* Catch exceptions. */
    if (local) {
        const char* msg = cgul_exception__get_message(local);
        fprintf(stderr, "*** Error: %s\n", msg);
        cgul_exception__delete(local);
        local = NULL;
        rv = 1;
    }

    /* Clean up. */
    if (utf32) {
        cgul_wstring__delete(utf32);
    }
    if (utf8) {
        cgul_string__delete(utf8);
    }

    return rv;
}
